// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
funcprot(0);
// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

function [x,y,z]=fSphereData()
    u=linspace(-%pi/2, %pi/2, 100);
    v=linspace(0, 2*%pi, 100);
    
    x=cos(u)'*cos(v);
    y=cos(u)'*sin(v);
    z=sin(u)'*ones(v);
endfunction

// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

// defines the color for each facet, considering dot product 
// of each direction the the light (cosang)

function [col]=setColor(cosang,icol)
  c1=-0.03; 
  c2=0.03; 
  c3=1;

  col=zeros(cosang);

  I=find(cosang<=c1);
  col(I)=icol(1); 

  I=find(cosang>c1 & cosang<c2);
  col(I)=icol(1)+round(((cosang(I)-c1)/(c2-c1))*(icol(2)-icol(1))); 
  // small middle zone
  
  I=find(cosang>=c2);
  col(I)=icol(2)+round(((cosang(I)-c2)/(c3-c2))*(icol(3)-icol(2))); 
  // yellow zone

endfunction

// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

// Test

[x,y,z]=fSphereData();

tic();
f=scf();
f.visible="off";
f.immediate_drawing="off"; 
clf(f);

plot3d2(x,y,z); 

e=gce();
fcData=e.data;

// The light comes from +z
// d.z is the scalar product of the vector with the direction 
// of the light => color

icol=1+[1,31,62]; 
sphereCol=setColor(fcData.z,icol);
   
TL=tlist(["3d" "x" "y" "z" "color"],..
         fcData.x,fcData.y,fcData.z,sphereCol);

e.data=TL;
e.color_flag=2; // 'flat' shading
e.color_mode=-1; // no edges
  
// graphic settings
a=gca();
a.axes_visible=["off","off","off"];
a.box="off"; 
a.x_label.text=""; 
a.y_label.text=""; 
a.z_label.text=""; 
a.isoview="on";

f.immediate_drawing="on"; 
f.visible="on"; 

dt=(0.8-toc())*1000;
sleep(max(dt,1)); // sleeps at most 0.8 s
